@@ -1,7 +1,7 @@
 /*
  *  declarations for encryption/security mechanisms for cryptmount
- *  $Revision: 130 $, $Date: 2006-10-15 11:42:36 +0100 (Sun, 15 Oct 2006) $
- *  Copyright 2005-2006, RW Penney
+ *  $Revision: 158 $, $Date: 2007-04-10 07:27:01 +0100 (Tue, 10 Apr 2007) $
+ *  Copyright 2005-2007, RW Penney
  */
 
 /*
@@ -32,24 +32,23 @@
 
 struct keyinfo;
 struct cment;
-
-void *sec_realloc(void *ptr, size_t size);
-void mem_cleanse(unsigned char *addr, size_t sz);
-void sec_free(void *ptr);
-
+struct cm_testinfo;
 
 typedef struct keymanager {
     const char *ident;
 
     int initialized;
-    int (*init_algs)();
-    int (*free_algs)();
+    int (*init_algs)(void);
+    int (*free_algs)(void);
 
     /* replace NULL algo-fields in keyinfo with suitable defaults: */
     void (*mk_default)(struct keyinfo *keyinfo);
 
     /* determine whether this keymanager can handle given key: */
-    int (*is_compat)(const struct keyinfo *keyinfo);
+    int (*is_compat)(const struct keyinfo *keyinfo, FILE *fp_key);
+
+    /* check if a password is needed to access the key: */
+    int (*needs_pw)(const struct keyinfo *keyinfo);
 
     /* extract encrypted key from file: */
     int (*get_key)(const char *ident, const struct keyinfo *keyinfo,
@@ -60,39 +59,62 @@ typedef struct keymanager {
                     const unsigned char *key, const int keylen, FILE *fp_key);
 
     /* message-digest computation (used as source of entropy): */
-    void* (*md_prepare)();
-    void (*md_block)(void *state, unsigned char *buff, unsigned len);
-    void (*md_final)(void *state, unsigned char **mdval, unsigned *mdlen);
+    void* (*md_prepare)(void);
+    void (*md_block)(void *state, unsigned char *buff, size_t len);
+    void (*md_final)(void *state, unsigned char **mdval, size_t *mdlen);
     void (*md_release)(void *state);
 
+    /* linked-list scaffolding: */
+    struct keymanager *next;
+
+#ifdef TESTING
+    void (*install_testctxt)(struct cm_testinfo *context);
+    int (*run_tests)(void);
+#endif
 } keymanager_t;
 
 
-keymanager_t *get_keymanager(const struct keyinfo *keyinfo);
+const char **get_keymgr_list(void);
+keymanager_t *get_keymanager(const struct keyinfo *keyinfo, FILE *fp_key);
 
 
-int init_algorithms();
-int free_algorithms();
-
+keymanager_t *kmblti_gethandle(void);
+keymanager_t *kmssl_gethandle(void);
+keymanager_t *kmgcry_gethandle(void);
 keymanager_t *init_keymanager(keymanager_t *km);
-int free_keymanagers();
+int free_keymanagers(void);
 
 int is_keysafe(const struct keyinfo *keyinfo);
-int get_randkey(unsigned char *key, unsigned len);
 int get_key(const char *ident, const struct keyinfo *keyinfo,
             unsigned char **key, int *keylen);
 int put_key(const char *ident, const struct keyinfo *keyinfo,
             const unsigned char *key, const int keylen, FILE *fp_key);
-int mk_key_string(const unsigned char *key, const int keylen, char *buff);
+size_t mk_key_string(const unsigned char *key, const size_t keylen,
+            char *buff);
 
+int sycheck_directory(const char *dirname);
 int sycheck_cmtab(const char *cmtab);
 int sycheck_target(const struct cment *ent);
 
-int cm_mutex_lock();
-int cm_mutex_unlock();
+int cm_mutex_lock(void);
+int cm_mutex_unlock(void);
+
+
+/* loadable keymanager-module helper macro: */
+#define KM_GETHANDLE(MOD_PATH, SYMBOL)                                      \
+    void *dlptr=NULL, *table=NULL;                                          \
+    dlptr = dlopen(MOD_PATH, RTLD_LAZY);                                    \
+    if (dlptr == NULL) {                                                    \
+        fprintf(stderr, "failed to open keymanager-module \"%s\" - %s\n",   \
+                MOD_PATH, dlerror());                                        \
+        return NULL;                                                        \
+    }                                                                       \
+    table = dlsym(dlptr, SYMBOL);                                           \
+    return (keymanager_t*)table;
+
 
 #endif  /* _ARMOUR_H */
 
 /*
- *  (C)Copyright 2005-2006, RW Penney
+ *  (C)Copyright 2005-2007, RW Penney
  */
