@@ -1,7 +1,7 @@
 /*
  *  methods for unit-testing utiltities for cryptmount
- *  $Revision: 130 $, $Date: 2006-10-15 11:42:36 +0100 (Sun, 15 Oct 2006) $
- *  Copyright 2006 RW Penney
+ *  $Revision: 158 $, $Date: 2007-04-10 07:27:01 +0100 (Tue, 10 Apr 2007) $
+ *  Copyright 2006-2007, RW Penney
  */
 
 /*
@@ -35,20 +35,17 @@
 #include "cmtesting.h"
 
 
-int cm_tests_run;
-int cm_tests_stat[CM_TEST_LAST];
-const char *cm_test_ident=NULL;
-
-void cm_tests_init();
-int cm_tests_close();
+static void cm_tests_init();
+static int cm_tests_close();
 
 
-void cm_tests_init()
+static void cm_tests_init()
 {   int i;
 
-    cm_tests_run = 0;
+    test_ctxtptr->current = NULL;
+    test_ctxtptr->tests_run = 0;
     for (i=0; i<CM_TEST_LAST; ++i) {
-        cm_tests_stat[i] = 0;
+        test_ctxtptr->test_stats[i] = 0;
     }
 }
 
@@ -57,26 +54,26 @@ int cm_tests_close()
 {   int i,nt;
 
     for (i=0,nt=0; i<CM_TEST_LAST; ++i) {
-        nt += cm_tests_stat[i];
+        nt += test_ctxtptr->test_stats[i];
     }
 
-    if (nt != cm_tests_run) {
+    if (nt != test_ctxtptr->tests_run) {
         fprintf(stderr, "mismatch in test-statistics (%d != %d)\n",
-                nt, cm_tests_run);
+                nt, test_ctxtptr->tests_run);
     }
 
-    if (cm_tests_stat[CM_TEST_PASSED] == cm_tests_run) {
-        fprintf(stderr, "++++ all %d tests PASSED ++++\n", cm_tests_run);
+    if (test_ctxtptr->test_stats[CM_TEST_PASSED] == test_ctxtptr->tests_run) {
+        fprintf(stderr, "++++ all %d tests PASSED ++++\n", test_ctxtptr->tests_run);
     } else {
         fprintf(stderr, "!!!! %2d tests FAILED !!!!\n",
-            cm_tests_stat[CM_TEST_FAILED]);
+            test_ctxtptr->test_stats[CM_TEST_FAILED]);
         fprintf(stderr, "!!!! %2d tests passed !!!!\n",
-            cm_tests_stat[CM_TEST_PASSED]);
+            test_ctxtptr->test_stats[CM_TEST_PASSED]);
         fprintf(stderr, "!!!! %2d tests aborted !!!!\n",
-            cm_tests_stat[CM_TEST_ABORTED]);
+            test_ctxtptr->test_stats[CM_TEST_ABORTED]);
     }
 
-    return (cm_tests_stat[CM_TEST_PASSED] != cm_tests_run);
+    return (test_ctxtptr->test_stats[CM_TEST_PASSED] != test_ctxtptr->tests_run);
 }
 
 
@@ -85,18 +82,11 @@ int cm_run_tests()
 {
     cm_tests_init();
 
-#if HAVE_OPENSSL
-    kmssl_init_algs();
-    kmssl_test_hash();
-    kmssl_test_getalgos();
-    kmssl_free_algs();
-#endif
-
-#if HAVE_LIBGCRYPT
-    kmgcry_test_hash();
-    kmgcry_test_getalgos();
-#endif
+    fs_test_blkgetsz();
+    ut_test_sha1();
+    bf_test_blowfish();
 
+    km_test_managers();
     km_test_keyrw();
 
     return cm_tests_close();
@@ -109,5 +99,5 @@ int _keep_ansi_pedantic_quiet = 0;
 #endif  /* TESTING */
 
 /*
- *  (C)Copyright 2006, RW Penney
+ *  (C)Copyright 2006-2007, RW Penney
  */
