@@ -1,7 +1,7 @@
 /*
  *  declarations for unit-test utilities for cryptmoumt
- *  $Revision: 130 $, $Date: 2006-10-15 11:42:36 +0100 (Sun, 15 Oct 2006) $
- *  Copyright 2006, RW Penney
+ *  $Revision: 158 $, $Date: 2007-04-10 07:27:01 +0100 (Tue, 10 Apr 2007) $
+ *  Copyright 2006-2007, RW Penney
  */
 
 /*
@@ -33,9 +33,6 @@
 #include <config.h>
 
 
-extern int cm_tests_run, cm_tests_stat[];
-extern const char *cm_test_ident;
-
 enum {
     CM_TEST_PASSED = 0,
     CM_TEST_FAILED,
@@ -44,58 +41,69 @@ enum {
 };
 
 
+typedef struct cm_testinfo {
+    const char *current;                /* name of current test */
+    int tests_run;                      /* total number of tests initiated */
+    int test_stats[CM_TEST_LAST];       /* statistics of test outcomes */
+
+    const char *argconfigdir;           /* adjustable config-directory */
+    const char *argpassword[2];         /* adjustable password arguments */
+} cm_testinfo_t;
+extern cm_testinfo_t *test_ctxtptr;
+
+
 #define CM_TEST_START(name) \
     { \
-        fprintf(stderr, "starting test %s...", name); \
-        ++cm_tests_run; \
-        cm_test_ident = NULL; \
+        fprintf(stderr, "starting test \"%s\"...", name); \
+        ++test_ctxtptr->tests_run; \
+        test_ctxtptr->current = NULL; \
     }
 
 #define CM_TEST_IDENT(ident) \
     { \
-        cm_test_ident = (const char*)ident; \
+        test_ctxtptr->current = (const char*)ident; \
     }
 
-#define CM_ASSERT_EQUAL(expected,actual) \
+#define CM_ASSERT_EQUAL(expected, actual) \
     if ((expected) != (actual)) { \
         fprintf(stderr, "  test failed %s:%d\n", __FILE__, __LINE__); \
-        if (cm_test_ident != NULL) { \
-            fprintf(stderr, "    [%s]\n", cm_test_ident); } \
-        ++cm_tests_stat[CM_TEST_FAILED]; \
+        if (test_ctxtptr->current != NULL) { \
+            fprintf(stderr, "    [%s]\n", test_ctxtptr->current); } \
+        ++test_ctxtptr->test_stats[CM_TEST_FAILED]; \
         return CM_TEST_FAILED; \
     }
 
-#define CM_ASSERT_DIFFERENT(expected,actual) \
+#define CM_ASSERT_DIFFERENT(expected, actual) \
     if ((expected) == (actual)) { \
         fprintf(stderr, "  test failed %s:%d\n", __FILE__, __LINE__); \
-        if (cm_test_ident != NULL) { \
-            fprintf(stderr, "    [%s]\n", cm_test_ident); } \
-        ++cm_tests_stat[CM_TEST_FAILED]; \
+        if (test_ctxtptr->current != NULL) { \
+            fprintf(stderr, "    [%s]\n", test_ctxtptr->current); } \
+        ++test_ctxtptr->test_stats[CM_TEST_FAILED]; \
         return CM_TEST_FAILED; \
     }
 
-#define CM_TEST_OK() \
+#define CM_TEST_OK(TI) \
     { \
         fprintf(stderr, " ok\n"); \
-        ++cm_tests_stat[CM_TEST_PASSED]; \
+        ++test_ctxtptr->test_stats[CM_TEST_PASSED]; \
         return CM_TEST_PASSED; \
     }
 
-#define CM_TEST_FAIL() \
+#define CM_TEST_FAIL(TI) \
     { \
         fprintf(stderr, " FAILED\n"); \
-        if (cm_test_ident != NULL) { \
-            fprintf(stderr, "    [%s]\n", cm_test_ident); } \
-        ++cm_tests_stat[CM_TEST_FAILED]; \
+        if (test_ctxtptr->current != NULL) { \
+            fprintf(stderr, "    [%s]\n", test_ctxtptr->current); } \
+        ++test_ctxtptr->test_stats[CM_TEST_FAILED]; \
         return CM_TEST_FAILED; \
     }
 
-#define CM_TEST_ABORT() \
+#define CM_TEST_ABORT(TI) \
     { \
         fprintf(stderr, " ABORTED\n"); \
-        if (cm_test_ident != NULL) { \
-            fprintf(stderr, "    [%s]\n", cm_test_ident); } \
-        ++cm_tests_stat[CM_TEST_ABORTED]; \
+        if (test_ctxtptr->current != NULL) { \
+            fprintf(stderr, "    [%s]\n", test_ctxtptr->current); } \
+        ++test_ctxtptr->test_stats[CM_TEST_ABORTED]; \
         return CM_TEST_ABORTED; \
     }
 
@@ -106,5 +114,5 @@ int cm_run_tests();
 #endif  /* _CMTEST_H */
 
 /*
- *  (C)Copyright 2006, RW Penney
+ *  (C)Copyright 2006-2007, RW Penney
  */
