@@ -1,7 +1,7 @@
 /*
  *  cryptmount - a utility for user-level mounting of encrypted filesystems
- *  $Revision: 130 $, $Date: 2006-10-15 11:42:36 +0100 (Sun, 15 Oct 2006) $
- *  Copyright 2005-2006 RW Penney
+ *  $Revision: 158 $, $Date: 2007-04-10 07:27:01 +0100 (Tue, 10 Apr 2007) $
+ *  Copyright 2005-2007 RW Penney
  */
 
 /*
@@ -45,6 +45,7 @@
 #ifdef HAVE_SYSLOG
 #  include <syslog.h>
 #endif
+#include <linux/fs.h>       /* beware ordering conflict with sys/mount.h */
 
 
 #include "armour.h"
@@ -54,6 +55,7 @@
 #include "fsutils.h"
 #include "looputils.h"
 #include "tables.h"
+#include "utils.h"
 #ifdef TESTING
 #  include "cmtesting.h"
 #endif
@@ -73,24 +75,26 @@ typedef enum
     M_PREPARE, M_RELEASE,
     M_MOUNT, M_UNMOUNT,
     M_SWAPON, M_SWAPOFF,
-    M_LIST, M_PASSWORD, M_KEYGEN, M_VERSION
+    M_LIST, M_KEYMGRS,
+    M_PASSWORD, M_KEYGEN, M_VERSION
 } cmmode_t;
 
 
-int execute_list(cmmode_t mode, const char *params,
+static int64_t getblkcount(const char *device);
+static int execute_list(cmmode_t mode, const char *params,
                 const targelt_t *eltlist);
-int do_list(const cment_t *cment);
-int do_devsetup(const cment_t *cment, char **mntdev);
-int do_devshutdown(const cment_t *cment);
-int do_mount(const cment_t *cment);
-int do_unmount(const cment_t *cment);
-int do_swapon(const cment_t *cment);
-int do_swapoff(const cment_t *cment);
-int do_passwd(const cment_t *cment);
-int do_keygen(const cment_t *cment, const char *params);
+static int do_list(const cment_t *cment);
+static int do_devsetup(const cment_t *cment, char **mntdev);
+static int do_devshutdown(const cment_t *cment);
+static int do_mount(const cment_t *cment);
+static int do_unmount(const cment_t *cment);
+static int do_swapon(const cment_t *cment);
+static int do_swapoff(const cment_t *cment);
+static int do_passwd(const cment_t *cment);
+static int do_keygen(const cment_t *cment, const char *params);
 
 
-static const char *USAGE_STRING=N_("\
+static const char *USAGE_STRING = N_("\
 usage: cryptmount [OPTION [target ...]]\n\
 \n\
   available options are as follows:\n\
@@ -98,6 +102,7 @@ usage: cryptmount [OPTION [target ...]]\n\
     -h | --help\n\
     -a | --all\n\
     -c | --change-password <target>\n\
+    -k | --key-managers\n\
     -l | --list\n\
     -m | --mount <target>\n\
     -u | --unmount <target>\n\
@@ -114,14 +119,95 @@ usage: cryptmount [OPTION [target ...]]\n\
 
 
 #ifdef TESTING
-/* allow config-file & password to be specified via command-line,
-    to enable automatic testing: */
-const char *argconfigdir=NULL;
-const char *argpassword[]={ NULL, NULL };
+
+cm_testinfo_t test_context;
+cm_testinfo_t *test_ctxtptr=&test_context;
+
+int fs_test_blkgetsz()
+    /* check that 32bit & 64bit device size calculations agree */
+{
+#ifdef BLKGETSIZE64
+    int fd, n_open, seclen, s0, s1, s2;
+    long len;
+    uint64_t len64;
+    const char **dev;
+    const char *devices[] = {
+        "/dev/hda", "/dev/hda1", "/dev/hda2", "/dev/hda3",
+        "/dev/hdb", "/dev/hdb1", "/dev/hdb2", "/dev/hdb3",
+        "/dev/hdc", "/dev/hdc1", "/dev/hdc2", "/dev/hdc3",
+        "/dev/sda", "/dev/sda1", "/dev/sda2", "/dev/sda3",
+        "/dev/scd", "/dev/scd1", "/dev/scd2", "/dev/scd3",
+        NULL };
 #endif
 
+    CM_TEST_START("BLKGETSIZE ioctl calls");
+#ifndef BLKGETSIZE64
+    /* assume that there is no ambiguity with BLKGETSIZE */
+    CM_TEST_PASS();
+#else
+    dev = devices;
+    n_open = 0;
+    while (*dev != NULL) {
+        fd = open(*dev, O_RDONLY);
+        if (fd >= 0) {
+            ++n_open;
+            if (ioctl(fd, BLKSSZGET, &seclen) != 0
+              || ioctl(fd, BLKGETSIZE, &len) != 0
+              || ioctl(fd, BLKGETSIZE64, &len64) != 0) {
+                CM_TEST_FAIL();
+            }
+            close(fd);
+            if (len64 < (1<<31)) {
+                CM_ASSERT_EQUAL(len64, ((int64_t)len * (int64_t)seclen));
+            }
+            /* fprintf(stderr, "%s: %d  %ld  %lld\n", *dev, seclen, len, len64 / seclen); */
+        }
+        ++dev;
+    }
 
-int do_list(const cment_t *cment)
+    if (n_open > 0) {
+        CM_TEST_OK();
+    } else {
+        CM_TEST_ABORT();
+    }
+#endif  /* BLKGETSIZE64 */
+}
+
+#endif  /* TESTING */
+
+
+static int64_t getblkcount(const char *device)
+    /* find size of raw device in blocks */
+{   int64_t count=-1;
+    int fd;
+#ifdef BLKGETSIZE64
+    int blklen;
+#else
+    long len;
+#endif
+
+    fd = open(device, O_RDONLY);
+    if (fd < 0) return (int64_t)-1;
+
+#ifdef BLKGETSIZE64
+    if (ioctl(fd, BLKGETSIZE64, &count) == 0
+        && ioctl(fd, BLKSSZGET, &blklen) == 0) {
+        count /= (int64_t)blklen;
+    } else {
+        count = -1;
+    }
+#else
+    if (ioctl(fd, BLKGETSIZE, &len) == 0) {
+        count = (int64_t)len;
+    }
+#endif
+
+    (void)close(fd);
+    return count;
+}
+
+
+static int do_list(const cment_t *cment)
     /* print list of available filing-system targets */
 {
     printf(_("%-16s  [to mount on \"%s\" as \"%s\"]\n"),
@@ -131,14 +217,15 @@ int do_list(const cment_t *cment)
 }
 
 
-int do_devsetup(const cment_t *cment, char **mntdev)
+static int do_devsetup(const cment_t *cment, char **mntdev)
     /* setup all devices needed to access encrypted target */
 {   enum { BUFFMIN=1024 };
     unsigned char *key=NULL;
-    int i,fd=-1,killloop=0,keylen=0,eflag=ERR_NOERROR;
-    long devlen=0,fslen=0;
+    int i, killloop=0, keylen=0, eflag=ERR_NOERROR;
+    int64_t devlen=0, fslen=0;
     struct stat sbuff;
-    char *dmparams=NULL,*tgtdev=NULL,*loopdev=NULL;
+    size_t dpsize;
+    char *dmparams=NULL, *tgtdev=NULL, *loopdev=NULL;
 
     /* get crypto-key for filing system: */
     eflag = get_key(cment->ident, &cment->key, &key, &keylen);
@@ -163,7 +250,7 @@ int do_devsetup(const cment_t *cment, char **mntdev)
             strcpy(loopdev, cment->loopdev);
         } else {
             loopdev = (char*)malloc((size_t)BUFFMIN);
-            if (loop_findfree(loopdev, BUFFMIN) != 0) {
+            if (loop_findfree(loopdev, (size_t)BUFFMIN) != 0) {
                 fprintf(stderr, _("no available loopback devices\n"));
                 eflag = ERR_BADDEVICE;
                 goto bail_out;
@@ -182,8 +269,8 @@ int do_devsetup(const cment_t *cment, char **mntdev)
     }
 
     /* get size in blocks of target device: */
-    fd = open(tgtdev, O_RDONLY);
-    if (ioctl(fd, BLKGETSIZE, &devlen) != 0) {
+    devlen = getblkcount(tgtdev);
+    if (devlen < 0) {
         fprintf(stderr, _("failed to get size of \"%s\"\n"), tgtdev);
         eflag = ERR_BADIOCTL;
         goto bail_out;
@@ -194,18 +281,20 @@ int do_devsetup(const cment_t *cment, char **mntdev)
         fslen = cment->length;
     }
     if (cment->start < 0 || fslen <= 0) {
-        fprintf(stderr, _("bad device-mapper start/length (%ld,%ld)\n"),
+        fprintf(stderr,_("bad device-mapper start/length"));
+        fprintf(stderr, " (%" PRId64 ",%" PRId64 ")\n",
                 cment->start, cment->length);
         eflag = ERR_BADDEVICE;
         goto bail_out;
     }
 
     /* setup device-mapper crypt table (CIPHER KEY IV_OFFSET DEV START): */
-    dmparams = sec_realloc(dmparams, (size_t)(2 * keylen + BUFFMIN));
-    i = sprintf(dmparams, "%s ",
+    dpsize = 2 * keylen + BUFFMIN;
+    dmparams = sec_realloc(dmparams, dpsize);
+    i = snprintf(dmparams, dpsize, "%s ",
                 (cment->cipher != NULL ? cment->cipher : DFLT_CIPHER));
-    mk_key_string(key, keylen, dmparams + i);
-    sprintf(dmparams + strlen(dmparams), " %ld %s %ld",
+    i += mk_key_string(key, (size_t)keylen, dmparams + i);
+    snprintf(dmparams + i, (dpsize - i), " %" PRId64 " %s %" PRId64,
                 cment->ivoffset, tgtdev, cment->start);
 
     /* setup device-mapper target: */
@@ -225,7 +314,6 @@ int do_devsetup(const cment_t *cment, char **mntdev)
 
   bail_out:
 
-    if (fd != -1) close(fd);
     if (loopdev != NULL) {
         if (killloop) loop_destroy(loopdev);    /* mounting failed? */
         free((void*)loopdev);
@@ -292,7 +380,7 @@ int do_devshutdown(const cment_t *cment)
 }
 
 
-int do_mount(const cment_t *cment)
+static int do_mount(const cment_t *cment)
 {   int freedev=0,eflag=ERR_NOERROR;
     char *mntdev=NULL;
     tgtstat_t *tstat;
@@ -337,7 +425,7 @@ int do_mount(const cment_t *cment)
 }
 
 
-int do_unmount(const cment_t *cment)
+static int do_unmount(const cment_t *cment)
 {   int eflag=ERR_NOERROR;
     struct passwd *pwent;
     char *mntdev=NULL;
@@ -386,7 +474,7 @@ int do_unmount(const cment_t *cment)
 }
 
 
-int do_swapon(const cment_t *cment)
+static int do_swapon(const cment_t *cment)
 {   int freedev=0,eflag=ERR_NOERROR;
     char *mntdev=NULL;
     tgtstat_t *tstat;
@@ -427,7 +515,7 @@ int do_swapon(const cment_t *cment)
 }
 
 
-int do_swapoff(const cment_t *cment)
+static int do_swapoff(const cment_t *cment)
 {   int eflag=ERR_NOERROR;
     char *mntdev=NULL;
     tgtstat_t *tstat;
@@ -469,7 +557,7 @@ int do_swapoff(const cment_t *cment)
 }
 
 
-int do_passwd(const cment_t *cment)
+static int do_passwd(const cment_t *cment)
 {   unsigned char *key=NULL;
     int keylen=0,eflag=ERR_NOERROR;
     char *newfname=NULL,*oldfname=NULL;
@@ -500,7 +588,7 @@ int do_passwd(const cment_t *cment)
         goto bail_out;
     }
     eflag = put_key(cment->ident, &cment->key, key, keylen, fp);
-    fclose(fp);
+    if (fclose(fp) != 0) eflag = ERR_BADFILE;
 
     if (eflag == ERR_NOERROR) {
         /* FIXME - check return codes */
@@ -529,9 +617,9 @@ int do_passwd(const cment_t *cment)
 }
 
 
-int do_keygen(const cment_t *cment, const char *params)
+static int do_keygen(const cment_t *cment, const char *params)
 {   unsigned char *key=NULL;
-    int keylen=0,eflag=ERR_NOERROR;
+    int keylen=0, eflag=ERR_NOERROR;
     char *fname=NULL;
     size_t sz;
     struct stat sbuff;
@@ -552,19 +640,12 @@ int do_keygen(const cment_t *cment, const char *params)
 
     fprintf(stderr, _("generating random key; please be patient...\n"));
     key = (unsigned char*)sec_realloc(NULL, (size_t)keylen);
-    eflag = get_randkey(key, (unsigned)keylen);
+    eflag = get_randkey(key, (size_t)keylen);
     if (eflag != ERR_NOERROR) {
         fprintf(stderr, _("failed to generate new key\n"));
         goto bail_out;
     }
 
-    if (!is_keysafe(&cment->key)) {
-        fprintf(stderr, _("key-file for \"%s\" isn't password-protected\n"),
-                cment->ident);
-        eflag = WRN_NOPASSWD;
-        goto bail_out;
-    }
-
     sz = strlen(cment->key.filename) + 16;
     fname = (char*)malloc(sz);
     snprintf(fname, sz, "%s-new", cment->key.filename);
@@ -640,6 +721,7 @@ static int check_mode(cmmode_t mode, const char *params,
         { M_SWAPON,     F_NEEDS_TGT },
         { M_SWAPOFF,    F_NEEDS_TGT },
         { M_LIST,       F_NORMALUSER },
+        { M_KEYMGRS,    F_NORMALUSER },
         { M_PASSWORD,   F_NORMALUSER | F_NEEDS_TGT },
         { M_KEYGEN,     F_NEEDS_TGT | F_NEEDS_PRM},
         { M_VERSION,    F_NORMALUSER },
@@ -672,12 +754,12 @@ static int check_mode(cmmode_t mode, const char *params,
 }
 
 
-int execute_list(cmmode_t mode, const char *params,
+static int execute_list(cmmode_t mode, const char *params,
                  const targelt_t *eltlist)
 {   const targelt_t *elt;
-    int prio=0,eflag=ERR_NOERROR;
-    struct passwd *pwent;
-    const char *username,*syslogmsg=NULL;
+    int i, prio=0, eflag=ERR_NOERROR;
+    struct passwd *pwent=NULL;
+    const char *username, **keymgrs, *syslogmsg=NULL;
 
     pwent = getpwuid(getuid());
     username = (pwent != NULL ? pwent->pw_name : "UNKNOWN");
@@ -692,6 +774,13 @@ int execute_list(cmmode_t mode, const char *params,
     /* execute user-selected task: */
     if (mode == M_VERSION) {
         fprintf(stderr, "%s-%s\n", PACKAGE_NAME, PACKAGE_VERSION);
+    } else if (mode == M_KEYMGRS) {
+        keymgrs = get_keymgr_list();
+        for (i=0; keymgrs[i]!=NULL; ) {
+            printf("%s", keymgrs[i]);
+            if (keymgrs[++i] != NULL) printf(", "); else printf("\n");
+        }
+        free((void*)keymgrs);
     } else {
         for (elt=eltlist; elt!=NULL && eflag<ERR_threshold; elt=elt->nx) {
             syslogmsg = NULL;
@@ -760,7 +849,7 @@ int execute_list(cmmode_t mode, const char *params,
 }
 
 
-cmmode_t get_defaultmode(int argc, char *argv[])
+static cmmode_t get_defaultmode(int argc, char *argv[])
     /* translate program-name into default action (or just assume M_MOUNT) */
 {   cmmode_t mode = M_MOUNT;
 
@@ -811,6 +900,7 @@ int main(int argc, char *argv[])
         { "config-fd",          required_argument,  NULL, (int)'f' },
         { "generate-key",       required_argument,  NULL, (int)'g' },
         { "help",               no_argument,        NULL, (int)'h' },
+        { "key-managers",       no_argument,        NULL, (int)'k' },
         { "list",               no_argument,        NULL, (int)'l' },
         { "mount",              no_argument,        NULL, (int)'m' },
         { "prepare",            no_argument,        NULL, (int)'p' },
@@ -827,7 +917,7 @@ int main(int argc, char *argv[])
 #  endif
         { NULL, 0, NULL, 0 } };
 #endif
-    const char *params=NULL, *optlist="acf:g:lmprsuvx";
+    const char *params=NULL, *optlist="acf:g:klmprsuvx";
     char *cmtab=NULL;
     int val,idx,config_fd=-1,eflag=ERR_NOERROR;
     cment_t *cmtable=NULL;
@@ -843,6 +933,7 @@ int main(int argc, char *argv[])
 
 #ifdef TESTING
     fprintf(stderr, "WARNING!!! cryptmount has been compiled for TESTING only - DO NOT INSTALL\n");
+    test_context.argconfigdir = test_context.argpassword[0] = test_context.argpassword[1] = NULL;
 #endif
 
     if (mlockall(MCL_CURRENT | MCL_FUTURE) != 0) {
@@ -868,6 +959,8 @@ int main(int argc, char *argv[])
                 mode = M_KEYGEN; params = optarg; break;
             case 'h':
                 mode = M_HELP; break;
+            case 'k':
+                mode = M_KEYMGRS; break;
             case 'l':
                 mode = M_LIST;  break;
             case 'm':
@@ -886,11 +979,11 @@ int main(int argc, char *argv[])
                 mode = M_VERSION; break;
 #ifdef TESTING
             case 'D':
-                argconfigdir = optarg; break;
+                test_context.argconfigdir = optarg; break;
             case 'W':
-                argpassword[0] = optarg; break;
+                test_context.argpassword[0] = optarg; break;
             case 'N':
-                argpassword[1] = optarg; break;
+                test_context.argpassword[1] = optarg; break;
             case 'T':
                 return cm_run_tests(); break;
 #endif
@@ -914,7 +1007,8 @@ int main(int argc, char *argv[])
 
     /* (check &) read-in configuration file: */
 #ifndef TESTING
-    if (sycheck_cmtab(cmtab) != ERR_NOERROR) {
+    if (sycheck_cmtab(cmtab) != ERR_NOERROR
+      || sycheck_directory(CM_MODULE_DIR) != ERR_NOERROR) {
         fprintf(stderr, _("security failure\n"));
         exit(EXIT_INSECURE);
     }
@@ -987,5 +1081,5 @@ int main(int argc, char *argv[])
 
 /*
  *  cryptmount.c
- *  (C)Copyright 2005-2006, RW Penney
+ *  (C)Copyright 2005-2007, RW Penney
  */
