@@ -1,7 +1,7 @@
 /*
  *  general declarations for cryptmount
- *  $Revision: 130 $, $Date: 2006-10-15 11:42:36 +0100 (Sun, 15 Oct 2006) $
- *  Copyright 2005-2006, RW Penney
+ *  $Revision: 158 $, $Date: 2007-04-10 07:27:01 +0100 (Tue, 10 Apr 2007) $
+ *  Copyright 2005-2007, RW Penney
  */
 
 /*
@@ -32,6 +32,21 @@
 
 #include <config.h>
 
+#if HAVE_INTTYPES_H
+#  include <inttypes.h>
+#else
+#  if HAVE_STDINT_H
+#    include <stdint.h>
+#  else
+     typedef long int32_t;
+     typedef unsigned long uint32_t;
+     typedef long long int64_t;
+     typedef unsigned long long uint64_t;
+#  endif
+#  define PRId64 "lld"
+#  define SCNi64 "lli"
+#endif
+
 #ifdef HAVE_GETTEXT 
 #  include <libintl.h>
 #  include <locale.h>
@@ -83,7 +98,8 @@ enum    /* error flags */
     ERR_BADSWAP,
     ERR_INSECURE,
     ERR_BADPASSWD,
-    ERR_BADPARAM
+    ERR_BADPARAM,
+    ERR_BADMUTEX
 };
 
 
@@ -113,14 +129,14 @@ typedef struct cment
     unsigned flags;         /* configuration switches */
 
     char *dev;              /* device node or raw file */
-    long start, length;     /* starting sector + num of sectors (or 0, -1) */
+    int64_t start, length;  /* starting sector + num of sectors (or 0, -1) */
     char *dir;              /* mount-point */
     char *fstype;           /* filesystem type */
     char *fsoptions;	    /* mount options */
     char *loopdev;	    /* loopback device to wrap around raw file */
 
     char *cipher;           /* cipher used on filesystem */
-    long ivoffset;          /* cipher initialization-vector offset */
+    int64_t ivoffset;       /* cipher initialization-vector offset */
 
     keyinfo_t key;          /* location/format of key */
 
@@ -130,5 +146,5 @@ typedef struct cment
 #endif  /* _CRYPTMOUNT_H */
 
 /*
- *  (C)Copyright 2005-2006, RW Penney
+ *  (C)Copyright 2005-2007, RW Penney
  */
