@@ -1,7 +1,7 @@
 /*
  *  filesystem-related utilities for cryptmount
- *  $Revision: 130 $, $Date: 2006-10-15 11:42:36 +0100 (Sun, 15 Oct 2006) $
- *  Copyright 2005-2006, RW Penney
+ *  $Revision: 158 $, $Date: 2007-04-10 07:27:01 +0100 (Tue, 10 Apr 2007) $
+ *  Copyright 2005-2007, RW Penney
  */
 
 /*
@@ -454,13 +454,13 @@ static int run_sucommand(const char *path, const char **argv,
             if ((switches & F_MATCHUID) != 0) {
                 /* change real UID to match effective UID
                    (probably only useful if euid==root): */
-                setuid(geteuid());
+                (void)setuid(geteuid());
             }
             if ((switches & F_CLOSE1) != 0) {
                 /* redirect standard output to /dev/null */
                 fd = open("/dev/null", O_WRONLY);
-                if (fd >= 0) dup2(fd, STDOUT_FILENO);
-                else close(STDOUT_FILENO);
+                if (fd >= 0) (void)dup2(fd, STDOUT_FILENO);
+                else (void)close(STDOUT_FILENO);
             }
 
             execv(path, (char *const *)argv);
@@ -477,9 +477,9 @@ static int run_sucommand(const char *path, const char **argv,
     return eflag;
 }
 
-#endif
+#endif  /* !ERSATZ_MOUNT ... */
 
 /*
  *  fsutils.c
- *  (C)Copyright 2005-2006, RW Penney
+ *  (C)Copyright 2005-2007, RW Penney
  */
