@@ -1,7 +1,7 @@
 /*
  *  loopback-device utilities for cryptmount
- *  $Revision: 130 $, $Date: 2006-10-15 11:42:36 +0100 (Sun, 15 Oct 2006) $
- *  Copyright 2005-2006, RW Penney
+ *  $Revision: 158 $, $Date: 2007-04-10 07:27:01 +0100 (Tue, 10 Apr 2007) $
+ *  Copyright 2005-2007, RW Penney
  */
 
 /*
@@ -71,16 +71,16 @@ static char *loop_formats[] = {
 #endif
 
 
-int loop_findfree(char *buff, int buffsz)
+int loop_findfree(char *buff, size_t buffsz)
     /* search for vacant loopback device */
-{   unsigned idx,min,found=0;
+{   unsigned idx, min, found=0;
     int devfd;
     my_loopinfo_t linfo;
     char str[256];
     struct stat sbuff;
 
     str[0] = '\0';
-    for (min=0; min<256 && !found; ++min) {
+    for (min=0; min<256u && !found; ++min) {
         for (idx=0; loop_formats[idx]!=NULL && !found; ++idx) {
             sprintf(str, loop_formats[idx], min);
 	    if (stat(str, &sbuff) || !S_ISBLK(sbuff.st_mode)) continue;
@@ -108,7 +108,7 @@ int loop_setup(const char *dev, const char *file, int flags)
     my_loopinfo_t lpinfo;
 
     memset((void*)&lpinfo, 0, sizeof(lpinfo));
-    strncpy((char*)LOOPFILENAME(lpinfo), file, LO_NAME_SIZE);
+    strncpy((char*)LOOPFILENAME(lpinfo), file, (size_t)LO_NAME_SIZE);
     lpinfo.lo_offset = 0;
     lpinfo.lo_encrypt_key_size = 0;
 
@@ -161,7 +161,7 @@ int loop_destroy(const char *dev)
 
   bail_out:
 
-    if (devfd >= 0) close(devfd);
+    if (devfd >= 0) (void)close(devfd);
 
     return eflag;
 }
@@ -189,5 +189,5 @@ int loop_ident(unsigned maj, unsigned min, char *buff, size_t buffsz)
 }
 
 /*
- *  (C)Copyright 2005-2006, RW Penney
+ *  (C)Copyright 2005-2007, RW Penney
  */
