@@ -1,7 +1,7 @@
 /*
  *  declarations for loopback-device utilities for cryptmount
- *  $Revision: 130 $, $Date: 2006-10-15 11:42:36 +0100 (Sun, 15 Oct 2006) $
- *  Copyright 2005, RW Penney
+ *  $Revision: 158 $, $Date: 2007-04-10 07:27:01 +0100 (Tue, 10 Apr 2007) $
+ *  Copyright 2005-2007, RW Penney
  */
 
 /*
@@ -31,7 +31,7 @@
 
 #include <stdlib.h>
 
-int loop_findfree(char *buff, int buffsz);
+int loop_findfree(char *buff, size_t buffsz);
 int loop_setup(const char *dev, const char *file, int flags);
 int loop_ident(unsigned maj, unsigned min, char *buff, size_t buffsz);
 int loop_destroy(const char *dev);
