@@ -1,7 +1,7 @@
 /*
  *  config-table and mount-table utilities for cryptmount
- *  $Revision: 130 $, $Date: 2006-10-15 11:42:36 +0100 (Sun, 15 Oct 2006) $
- *  Copyright 2005-2006, RW Penney
+ *  $Revision: 158 $, $Date: 2007-04-10 07:27:01 +0100 (Tue, 10 Apr 2007) $
+ *  Copyright 2005-2007, RW Penney
  */
 
 /*
@@ -40,6 +40,10 @@
 #include "cryptmount.h"
 #include "delegates.h"
 #include "tables.h"
+#include "utils.h"
+#ifdef TESTING
+#  include "cmtesting.h"
+#endif
 
 
 enum                /* config-file tokens */
@@ -60,41 +64,7 @@ struct statfile     /* information about status file */
 static cment_t *parse_stream(FILE *fp, const char *cfgname);
 
 
-int cm_path(char **buff, const char *file)
-    /* create full pathname of config file */
-{   size_t pfxlen, sfxlen;
-    const char *pfx=NULL;
-
-    if (buff == NULL || file == NULL) return 0;
-
-#ifdef TESTING
-    extern char *argconfigdir;
-    pfx = (argconfigdir != NULL ? argconfigdir : "/nowhere");
-#else
-    pfx = CM_CONFIG_DIR;
-#endif
-
-    pfxlen = strlen(pfx);
-    sfxlen = strlen(file);
-    *buff = (char*)realloc((void*)(*buff), (pfxlen + sfxlen + 2));
-
-    snprintf(*buff, (pfxlen + sfxlen + 2), "%s/%s", pfx, file);
-
-    return (int)(pfxlen + sfxlen + 1);
-}
-
-
-char *cm_strdup(const char *orig)
-    /* make duplicate of existing string, reserving memory */
-{   char *cpy;
-
-    if (orig == NULL) return NULL;
-    cpy = (char*)malloc((size_t)(strlen(orig) + 1));
-    return strcpy(cpy, orig);
-}
-
-
-cment_t *alloc_cment()
+cment_t *alloc_cment(void)
     /* allocate storage for a target-structure */
 {   cment_t *cment;
 
@@ -117,7 +87,7 @@ cment_t *alloc_cment()
     cment->key.filename = NULL;
     cment->key.digestalg = NULL;
     cment->key.cipheralg = NULL;
-    cment->key.maxlen = -1;
+    cment->key.maxlen = -1L;
 
     cment->nx = NULL;
 
@@ -189,7 +159,7 @@ static int proc_string(void *ptr, const char *src)
 
 static int proc_long(void *ptr, const char *src)
     /* process var=val entry in config-table for "long" type */
-{   long ql,*addr=(long*)ptr;
+{   long ql, *addr=(long*)ptr;
 
     if (sscanf(src, "%ld", &ql) == 1) {
         *addr = ql;
@@ -200,6 +170,19 @@ static int proc_long(void *ptr, const char *src)
 }
 
 
+static int proc_int64(void *ptr, const char *src)
+    /* process var=val entry in config-table for "int64" type */
+{   int64_t ql, *addr=(int64_t*)ptr;
+
+    if (sscanf(src, "%" SCNi64, &ql) == 1) {
+        *addr = ql;
+        return 0;
+    }
+
+    return 1;
+}
+
+
 int proc_flags(void *ptr, const char *src)
     /* convert string of configuration-switches into binary flags */
 {   unsigned *flags=(unsigned*)ptr;
@@ -253,13 +236,13 @@ static void read_token(char *buff, unsigned *t_state, cment_t *cment)
         { "flags",          OFFSET(flags),          proc_flags },
         { "dev",            OFFSET(dev),            proc_string },
         { "dir",            OFFSET(dir),            proc_string },
-        { "startsector",    OFFSET(start),          proc_long },
-        { "numsectors",     OFFSET(length),         proc_long },
+        { "startsector",    OFFSET(start),          proc_int64 },
+        { "numsectors",     OFFSET(length),         proc_int64 },
         { "fstype",         OFFSET(fstype),         proc_string },
         { "fsoptions",      OFFSET(fsoptions),      proc_string },
         { "loop",           OFFSET(loopdev),        proc_string },
         { "cipher",         OFFSET(cipher),         proc_string },
-        { "ivoffset",       OFFSET(ivoffset),       proc_long },
+        { "ivoffset",       OFFSET(ivoffset),       proc_int64 },
         { "keyformat",      OFFSET(key.format),     proc_string },
         { "keyfile",        OFFSET(key.filename),   proc_string },
         { "keyhash",        OFFSET(key.digestalg),  proc_string },
@@ -280,6 +263,12 @@ static void read_token(char *buff, unsigned *t_state, cment_t *cment)
             break;
         case T_OPT:
             if (strcmp(buff, "}") == 0) {
+                if (cment->key.format == NULL
+                  && cment->key.cipheralg != NULL
+                  && strcmp(cment->key.cipheralg, "none") == 0) {
+                    fprintf(stderr, _("cryptmount: using \"keycipher=none\" is deprecated - please use \"keyformat=raw\" instead\n"));
+                    cment->key.format = cm_strdup("raw");
+                }
                 *t_state = T_RBRACE; break;
             }
             if ((eq = strchr(buff,'=')) == NULL) {
@@ -559,7 +548,7 @@ int put_tgtstatus(const cment_t *cment, const tgtstat_t *newstat)
     struct statfile *sfin,*sfout;
     tgtstat_t *ts=NULL;
     struct stat sbuff;
-    int badlock,eflag=0;
+    int badlock, eflag=0;
 
     (void)cm_path(&oldfname,"cmstatus");
     (void)cm_path(&newfname,"cmstatus-temp");
@@ -611,5 +600,5 @@ int put_tgtstatus(const cment_t *cment, const tgtstat_t *newstat)
 
 /*
  *  tables.c
- *  (C)Copyright 2005-2006, RW Penney
+ *  (C)Copyright 2005-2007, RW Penney
  */
